#include <random>

#include "BinaryTreeAlgorithm.h"
#include "CartesianCell.h"
#include "CartesianGrid.h"

CartesianGrid BinaryTreeAlgorithm::Build(int rows, int cols) {
  CartesianGrid grid(rows, cols);

  std::mt19937 rng((std::random_device())());
  std::uniform_int_distribution<int> range(1, 2);
  std::function<int()> random(std::bind(range, rng));

  // // Lambda definition
  auto cellFunction = [random](CartesianCell* cellptr) {
    // Collect north and east, if they are valid
    CartesianCell* north = cellptr->north();
    CartesianCell* east = cellptr->east();
    std::vector<CartesianCell*> neighbors;
    if (north != nullptr) {
      neighbors.push_back(north);
    }
    if (east != nullptr) {
      neighbors.push_back(east);
    }
    // If we have neighbors pick a valid one randomly and link it
    if (neighbors.size() > 0) {
      int index((random() % neighbors.size()));
      CartesianCell* neighbor = neighbors[index];
      cellptr->link(neighbor);
    }
  };

  // // Apply lambda to all cells
  grid.foreachCell(cellFunction);

  return grid;
}