#include <algorithm>
#include <memory>

#include "CartesianCell.h"

CartesianCell::CartesianCell(int row, int col)
    : mrow(row), mcol(col), mlinks(), mneighbors(4) {}

bool CartesianCell::operator==(const CartesianCell& other) const {
  return mrow == other.mrow && mcol == other.mcol;
}

bool CartesianCell::isLinked(CartesianCell* cell) const {
  if (cell == nullptr) return false;
  auto linksEnd = std::end(mlinks);
  return linksEnd != std::find_if(std::begin(mlinks), linksEnd,
                                  [cell](CartesianCell* cellptr) {
                                    return (*cellptr) == (*cell);
                                  });
}

void CartesianCell::link(CartesianCell* cell, bool bidir) {
  if (cell == nullptr) return;
  if (!isLinked(cell)) {
    mlinks.push_back(cell);
  }
  if (bidir) {
    cell->link(this, false);
  }
}

void CartesianCell::unlink(CartesianCell* cell, bool bidir) {
  if (cell == nullptr) return;
  auto removeIterator = std::remove_if(
      std::begin(mlinks), std::end(mlinks),
      [cell](CartesianCell* cellptr) { return (*cell) == (*cellptr); });
  mlinks.erase(removeIterator);
  if (bidir) {
    cell->unlink(this, false);
  }
}

int CartesianCell::row() { return mrow; }
int CartesianCell::col() { return mcol; }

void CartesianCell::north(CartesianCell* cell) { mneighbors[0] = cell; }
void CartesianCell::east(CartesianCell* cell) { mneighbors[1] = cell; }
void CartesianCell::south(CartesianCell* cell) { mneighbors[2] = cell; }
void CartesianCell::west(CartesianCell* cell) { mneighbors[3] = cell; }
CartesianCell* CartesianCell::north() { return mneighbors[0]; }
CartesianCell* CartesianCell::east() { return mneighbors[1]; }
CartesianCell* CartesianCell::south() { return mneighbors[2]; }
CartesianCell* CartesianCell::west() { return mneighbors[3]; }